@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2011 inria.  All rights reserved.
+ * Copyright © 2009-2012 Inria.  All rights reserved.
  * Copyright © 2009-2012 Université Bordeaux 1
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -172,6 +172,18 @@ typedef struct _PSAPI_WORKING_SET_EX_INFORMATION {
 } PSAPI_WORKING_SET_EX_INFORMATION;
 #endif
 
+static void hwloc_bitmap_set_ith_ULONG_PTR(hwloc_bitmap_t set, unsigned i, ULONG_PTR mask)
+{
+	/* ULONG_PTR is 64/32bits depending on the arch
+	 * while unsigned long is always 32bits */
+#if SIZEOF_VOID_P == 8
+	hwloc_bitmap_set_ith_ulong(set, 2*i, mask & 0xffffffff);
+	hwloc_bitmap_set_ith_ulong(set, 2*i+1, mask >> 32);
+#else
+	hwloc_bitmap_set_ith_ulong(set, i, mask);
+#endif
+}
+
 /* TODO: SetThreadIdealProcessor */
 
 static int
@@ -307,9 +319,9 @@ hwloc_win_get_thisproc_membind(hwloc_topology_t topology, hwloc_nodeset_t nodese
   return hwloc_win_get_proc_membind(topology, GetCurrentProcess(), nodeset, policy, flags);
 }
 
-static LPVOID WINAPI (*VirtualAllocExNumaProc)(HANDLE hProcess, LPVOID lpAddress, SIZE_T dwSize, DWORD flAllocationType, DWORD flProtect, DWORD nndPreferred);
-static BOOL WINAPI (*VirtualFreeExProc)(HANDLE hProcess, LPVOID lpAddress, SIZE_T dwSize, DWORD dwFreeType);
-static BOOL WINAPI (*QueryWorkingSetExProc)(HANDLE hProcess, PVOID pv, DWORD cb);
+static LPVOID (WINAPI *VirtualAllocExNumaProc)(HANDLE hProcess, LPVOID lpAddress, SIZE_T dwSize, DWORD flAllocationType, DWORD flProtect, DWORD nndPreferred);
+static BOOL (WINAPI *VirtualFreeExProc)(HANDLE hProcess, LPVOID lpAddress, SIZE_T dwSize, DWORD dwFreeType);
+static BOOL (WINAPI *QueryWorkingSetExProc)(HANDLE hProcess, PVOID pv, DWORD cb);
 
 static int hwloc_win_get_VirtualAllocExNumaProc(void) {
   if (VirtualAllocExNumaProc == NULL) {
@@ -460,19 +472,26 @@ hwloc_win_get_area_membind(hwloc_topology_t topology __hwloc_attribute_unused, c
   }
 }
 
-void
-hwloc_look_windows(struct hwloc_topology *topology)
+static int
+hwloc_look_windows(struct hwloc_backend *backend)
 {
-  BOOL WINAPI (*GetLogicalProcessorInformationProc)(PSYSTEM_LOGICAL_PROCESSOR_INFORMATION Buffer, PDWORD ReturnLength);
-  BOOL WINAPI (*GetLogicalProcessorInformationExProc)(LOGICAL_PROCESSOR_RELATIONSHIP relationship, PSYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX Buffer, PDWORD ReturnLength);
-  BOOL WINAPI (*GetNumaAvailableMemoryNodeProc)(UCHAR Node, PULONGLONG AvailableBytes);
-  BOOL WINAPI (*GetNumaAvailableMemoryNodeExProc)(USHORT Node, PULONGLONG AvailableBytes);
+  struct hwloc_topology *topology = backend->topology;
+  BOOL (WINAPI *GetLogicalProcessorInformationProc)(PSYSTEM_LOGICAL_PROCESSOR_INFORMATION Buffer, PDWORD ReturnLength);
+  BOOL (WINAPI *GetLogicalProcessorInformationExProc)(LOGICAL_PROCESSOR_RELATIONSHIP relationship, PSYSTEM_LOGICAL_PROCESSOR_INFORMATION_EX Buffer, PDWORD ReturnLength);
+  BOOL (WINAPI *GetNumaAvailableMemoryNodeProc)(UCHAR Node, PULONGLONG AvailableBytes);
+  BOOL (WINAPI *GetNumaAvailableMemoryNodeExProc)(USHORT Node, PULONGLONG AvailableBytes);
   SYSTEM_INFO SystemInfo;
 
   DWORD length;
 
   HMODULE kernel32;
 
+  if (topology->levels[0][0]->cpuset)
+    /* somebody discovered things */
+    return 0;
+
+  hwloc_alloc_obj_cpusets(topology->levels[0][0]);
+
   GetSystemInfo(&SystemInfo);
 
   kernel32 = LoadLibrary("kernel32.dll");
@@ -496,16 +515,19 @@ hwloc_look_windows(struct hwloc_topology *topology)
 	if (GetLogicalProcessorInformationProc(procInfo, &length))
 	  break;
 	if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
-	  return;
+	  return -1;
 	procInfo = realloc(procInfo, length);
       }
 
+      assert(!length || procInfo);
+
       for (i = 0; i < length / sizeof(*procInfo); i++) {
 
-        /* Ignore non-data caches */
-	if (procInfo[i].Relationship == RelationCache
-		&& procInfo[i].Cache.Type != CacheUnified
-		&& procInfo[i].Cache.Type != CacheData)
+        /* Ignore unknown caches */
+	if (procInfo->Relationship == RelationCache
+		&& procInfo->Cache.Type != CacheUnified
+		&& procInfo->Cache.Type != CacheData
+		&& procInfo->Cache.Type != CacheInstruction)
 	  continue;
 
 	id = -1;
@@ -532,14 +554,8 @@ hwloc_look_windows(struct hwloc_topology *topology)
 	obj = hwloc_alloc_setup_object(type, id);
         obj->cpuset = hwloc_bitmap_alloc();
 	hwloc_debug("%s#%u mask %lx\n", hwloc_obj_type_string(type), id, procInfo[i].ProcessorMask);
-	/* ProcessorMask is a ULONG_PTR (64/32bits depending on the arch)
-	 * while unsigned long is always 32bits */
-#if SIZEOF_VOID_P == 8
-	hwloc_bitmap_set_ith_ulong(obj->cpuset, 0, procInfo[i].ProcessorMask & 0xffffffff);
-	hwloc_bitmap_set_ith_ulong(obj->cpuset, 1, procInfo[i].ProcessorMask >> 32);
-#else
-	hwloc_bitmap_set_ith_ulong(obj->cpuset, 0, procInfo[i].ProcessorMask);
-#endif
+	/* ProcessorMask is a ULONG_PTR */
+	hwloc_bitmap_set_ith_ULONG_PTR(obj->cpuset, 0, procInfo[i].ProcessorMask);
 	hwloc_debug_2args_bitmap("%s#%u bitmap %s\n", hwloc_obj_type_string(type), id, obj->cpuset);
 
 	switch (type) {
@@ -567,6 +583,20 @@ hwloc_look_windows(struct hwloc_topology *topology)
 	    obj->attr->cache.associativity = procInfo[i].Cache.Associativity == CACHE_FULLY_ASSOCIATIVE ? -1 : procInfo[i].Cache.Associativity ;
 	    obj->attr->cache.linesize = procInfo[i].Cache.LineSize;
 	    obj->attr->cache.depth = procInfo[i].Cache.Level;
+	    switch (procInfo->Cache.Type) {
+	      case CacheUnified:
+		obj->attr->cache.type = HWLOC_OBJ_CACHE_UNIFIED;
+		break;
+	      case CacheData:
+		obj->attr->cache.type = HWLOC_OBJ_CACHE_DATA;
+		break;
+	      case CacheInstruction:
+		obj->attr->cache.type = HWLOC_OBJ_CACHE_INSTRUCTION;
+		break;
+	      default:
+		hwloc_free_unlinked_object(obj);
+		continue;
+	    }
 	    break;
 	  case HWLOC_OBJ_GROUP:
 	    obj->attr->group.depth = procInfo[i].Relationship == RelationGroup;
@@ -594,7 +624,7 @@ hwloc_look_windows(struct hwloc_topology *topology)
 	if (GetLogicalProcessorInformationExProc(RelationAll, procInfoTotal, &length))
 	  break;
 	if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
-	  return;
+	  return -1;
         procInfoTotal = realloc(procInfoTotal, length);
       }
 
@@ -604,10 +634,11 @@ hwloc_look_windows(struct hwloc_topology *topology)
         unsigned num, i;
         GROUP_AFFINITY *GroupMask;
 
-        /* Ignore non-data caches */
+        /* Ignore unknown caches */
 	if (procInfo->Relationship == RelationCache
 		&& procInfo->Cache.Type != CacheUnified
-		&& procInfo->Cache.Type != CacheData)
+		&& procInfo->Cache.Type != CacheData
+		&& procInfo->Cache.Type != CacheInstruction)
 	  continue;
 
 	id = -1;
@@ -642,14 +673,8 @@ hwloc_look_windows(struct hwloc_topology *topology)
 	      mask = procInfo->Group.GroupInfo[id].ActiveProcessorMask;
 	      hwloc_debug("group %u %d cpus mask %lx\n", id,
                   procInfo->Group.GroupInfo[id].ActiveProcessorCount, mask);
-	      /* KAFFINITY is ULONG_PTR (64/32bits depending on the arch)
-	       * while unsigned long is always 32bits */
-#if SIZEOF_VOID_P == 8
-	      hwloc_bitmap_set_ith_ulong(obj->cpuset, 2*id, mask & 0xffffffff);
-	      hwloc_bitmap_set_ith_ulong(obj->cpuset, 2*id+1, mask >> 32);
-#else
-	      hwloc_bitmap_set_ith_ulong(obj->cpuset, id, mask);
-#endif
+	      /* KAFFINITY is ULONG_PTR */
+	      hwloc_bitmap_set_ith_ULONG_PTR(obj->cpuset, id, mask);
 	      hwloc_debug_2args_bitmap("group %u %d bitmap %s\n", id, procInfo->Group.GroupInfo[id].ActiveProcessorCount, obj->cpuset);
 	      hwloc_insert_object_by_cpuset(topology, obj);
 	    }
@@ -664,14 +689,8 @@ hwloc_look_windows(struct hwloc_topology *topology)
         obj->cpuset = hwloc_bitmap_alloc();
         for (i = 0; i < num; i++) {
           hwloc_debug("%s#%u %d: mask %d:%lx\n", hwloc_obj_type_string(type), id, i, GroupMask[i].Group, GroupMask[i].Mask);
-	  /* GROUP_AFFINITY.Mask is KAFFINITY, which is ULONG_PTR (64/32bits depending on the arch)
-	   * while unsigned long is always 32bits */
-#if SIZEOF_VOID_P == 8
-          hwloc_bitmap_set_ith_ulong(obj->cpuset, 2*GroupMask[i].Group, GroupMask[i].Mask & 0xffffffff);
-          hwloc_bitmap_set_ith_ulong(obj->cpuset, 2*GroupMask[i].Group+1, GroupMask[i].Mask >> 32);
-#else
-          hwloc_bitmap_set_ith_ulong(obj->cpuset, GroupMask[i].Group, GroupMask[i].Mask);
-#endif
+	  /* GROUP_AFFINITY.Mask is KAFFINITY, which is ULONG_PTR */
+	  hwloc_bitmap_set_ith_ULONG_PTR(obj->cpuset, GroupMask[i].Group, GroupMask[i].Mask);
         }
 	hwloc_debug("%s#%u bitmap %s\n", hwloc_obj_type_string(type), id, obj->cpuset);
 
@@ -700,6 +719,20 @@ hwloc_look_windows(struct hwloc_topology *topology)
 	    obj->attr->cache.associativity = procInfo->Cache.Associativity == CACHE_FULLY_ASSOCIATIVE ? -1 : procInfo->Cache.Associativity ;
 	    obj->attr->cache.linesize = procInfo->Cache.LineSize;
 	    obj->attr->cache.depth = procInfo->Cache.Level;
+	    switch (procInfo->Cache.Type) {
+	      case CacheUnified:
+		obj->attr->cache.type = HWLOC_OBJ_CACHE_UNIFIED;
+		break;
+	      case CacheData:
+		obj->attr->cache.type = HWLOC_OBJ_CACHE_DATA;
+		break;
+	      case CacheInstruction:
+		obj->attr->cache.type = HWLOC_OBJ_CACHE_INSTRUCTION;
+		break;
+	      default:
+		hwloc_free_unlinked_object(obj);
+		continue;
+	    }
 	    break;
 	  default:
 	    break;
@@ -714,32 +747,66 @@ hwloc_look_windows(struct hwloc_topology *topology)
   hwloc_setup_pu_level(topology, hwloc_fallback_nbprocessors(topology));
 
   hwloc_obj_add_info(topology->levels[0][0], "Backend", "Windows");
+  if (topology->is_thissystem)
+    hwloc_add_uname_info(topology);
+  return 1;
 }
 
 void
-hwloc_set_windows_hooks(struct hwloc_topology *topology)
+hwloc_set_windows_hooks(struct hwloc_binding_hooks *hooks,
+			struct hwloc_topology_support *support)
 {
-  topology->set_proc_cpubind = hwloc_win_set_proc_cpubind;
-  topology->get_proc_cpubind = hwloc_win_get_proc_cpubind;
-  topology->set_thread_cpubind = hwloc_win_set_thread_cpubind;
-  topology->set_thisproc_cpubind = hwloc_win_set_thisproc_cpubind;
-  topology->get_thisproc_cpubind = hwloc_win_get_thisproc_cpubind;
-  topology->set_thisthread_cpubind = hwloc_win_set_thisthread_cpubind;
+  hooks->set_proc_cpubind = hwloc_win_set_proc_cpubind;
+  hooks->get_proc_cpubind = hwloc_win_get_proc_cpubind;
+  hooks->set_thread_cpubind = hwloc_win_set_thread_cpubind;
+  hooks->set_thisproc_cpubind = hwloc_win_set_thisproc_cpubind;
+  hooks->get_thisproc_cpubind = hwloc_win_get_thisproc_cpubind;
+  hooks->set_thisthread_cpubind = hwloc_win_set_thisthread_cpubind;
   /* TODO: get_last_cpu_location: use GetCurrentProcessorNumber */
 
-  topology->set_proc_membind = hwloc_win_set_proc_membind;
-  topology->get_proc_membind = hwloc_win_get_proc_membind;
-  topology->set_thisproc_membind = hwloc_win_set_thisproc_membind;
-  topology->get_thisproc_membind = hwloc_win_get_thisproc_membind;
-  topology->set_thisthread_membind = hwloc_win_set_thisthread_membind;
+  hooks->set_proc_membind = hwloc_win_set_proc_membind;
+  hooks->get_proc_membind = hwloc_win_get_proc_membind;
+  hooks->set_thisproc_membind = hwloc_win_set_thisproc_membind;
+  hooks->get_thisproc_membind = hwloc_win_get_thisproc_membind;
+  hooks->set_thisthread_membind = hwloc_win_set_thisthread_membind;
 
   if (!hwloc_win_get_VirtualAllocExNumaProc()) {
-    topology->alloc_membind = hwloc_win_alloc_membind;
-    topology->alloc = hwloc_win_alloc;
-    topology->free_membind = hwloc_win_free_membind;
-    topology->support.membind->bind_membind = 1;
+    hooks->alloc_membind = hwloc_win_alloc_membind;
+    hooks->alloc = hwloc_win_alloc;
+    hooks->free_membind = hwloc_win_free_membind;
+    support->membind->bind_membind = 1;
   }
 
   if (!hwloc_win_get_QueryWorkingSetExProc())
-    topology->get_area_membind = hwloc_win_get_area_membind;
+    hooks->get_area_membind = hwloc_win_get_area_membind;
 }
+
+static struct hwloc_backend *
+hwloc_windows_component_instantiate(struct hwloc_disc_component *component,
+				    const void *_data1 __hwloc_attribute_unused,
+				    const void *_data2 __hwloc_attribute_unused,
+				    const void *_data3 __hwloc_attribute_unused)
+{
+  struct hwloc_backend *backend;
+  backend = hwloc_backend_alloc(component);
+  if (!backend)
+    return NULL;
+  backend->discover = hwloc_look_windows;
+  return backend;
+}
+
+static struct hwloc_disc_component hwloc_windows_disc_component = {
+  HWLOC_DISC_COMPONENT_TYPE_CPU,
+  "windows",
+  HWLOC_DISC_COMPONENT_TYPE_GLOBAL,
+  hwloc_windows_component_instantiate,
+  50,
+  NULL
+};
+
+const struct hwloc_component hwloc_windows_component = {
+  HWLOC_COMPONENT_ABI,
+  HWLOC_COMPONENT_TYPE_DISC,
+  0,
+  &hwloc_windows_disc_component
+};
