@@ -1,7 +1,7 @@
 /*
  * Copyright © 2009 CNRS
  * Copyright © 2009-2011 inria.  All rights reserved.
- * Copyright © 2009 Université Bordeaux 1
+ * Copyright © 2009, 2012 Université Bordeaux 1
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
@@ -29,6 +29,15 @@ int main(void)
   assert(hwloc_bitmap_to_ith_ulong(set, 0) == 0UL);
   assert(hwloc_bitmap_to_ith_ulong(set, 1) == 0UL);
   assert(hwloc_bitmap_to_ith_ulong(set, 23) == 0UL);
+  /* check a two-long bitmap */
+  hwloc_bitmap_from_ith_ulong(set, 4, 0xfe);
+  hwloc_bitmap_set_ith_ulong(set, 6, 0xef);
+  assert(hwloc_bitmap_to_ith_ulong(set, 4) == 0xfe);
+  assert(hwloc_bitmap_to_ith_ulong(set, 6) == 0xef);
+  assert(hwloc_bitmap_to_ulong(set) == 0UL);
+  assert(hwloc_bitmap_to_ith_ulong(set, 0) == 0UL);
+  assert(hwloc_bitmap_to_ith_ulong(set, 1) == 0UL);
+  assert(hwloc_bitmap_to_ith_ulong(set, 23) == 0UL);
   /* check a zeroed bitmap */
   hwloc_bitmap_zero(set);
   assert(hwloc_bitmap_to_ulong(set) == 0UL);
