@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2010 inria.  All rights reserved.
+ * Copyright © 2009-2013 Inria.  All rights reserved.
  * Copyright © 2009-2012 Université Bordeaux 1
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -46,12 +46,12 @@
  */
 
 static void
-output_console_obj (hwloc_obj_t l, FILE *output, int logical, int verbose_mode)
+output_console_obj (hwloc_topology_t topology, hwloc_obj_t l, FILE *output, int logical, int verbose_mode)
 {
   char type[32], *attr, phys[32] = "";
   unsigned idx = logical ? l->logical_index : l->os_index;
   const char *indexprefix = logical ? " L#" :  " P#";
-  if (show_cpuset < 2) {
+  if (lstopo_show_cpuset < 2) {
     int len;
     if (l->type == HWLOC_OBJ_MISC && l->name)
       fprintf(output, "%s", l->name);
@@ -81,17 +81,27 @@ output_console_obj (hwloc_obj_t l, FILE *output, int logical, int verbose_mode)
   }
   if (!l->cpuset)
     return;
-  if (show_cpuset == 1)
+  if (lstopo_show_cpuset == 1)
     fprintf(output, " cpuset=");
-  if (show_cpuset) {
+  if (lstopo_show_cpuset) {
     char *cpusetstr;
-    if (taskset)
+    if (lstopo_show_taskset)
       hwloc_bitmap_taskset_asprintf(&cpusetstr, l->cpuset);
     else
       hwloc_bitmap_asprintf(&cpusetstr, l->cpuset);
     fprintf(output, "%s", cpusetstr);
     free(cpusetstr);
   }
+
+  /* annotate if the PU is forbidden/offline/running */
+  if (l->type == HWLOC_OBJ_PU && verbose_mode >= 2) {
+    if (lstopo_pu_offline(l))
+      printf(" (offline)");
+    else if (lstopo_pu_forbidden(l))
+      printf(" (forbidden)");
+    else if (lstopo_pu_running(topology, l))
+      printf(" (running)");
+  }
 }
 
 /* Recursively output topology in a console fashion */
@@ -99,7 +109,7 @@ static void
 output_topology (hwloc_topology_t topology, hwloc_obj_t l, hwloc_obj_t parent, FILE *output, int i, int logical, int verbose_mode)
 {
   unsigned x;
-  int group_identical = (verbose_mode <= 1) && !show_cpuset;
+  int group_identical = (verbose_mode <= 1) && !lstopo_show_cpuset;
   if (group_identical
       && parent && parent->arity == 1
       && l->cpuset && parent->cpuset && hwloc_bitmap_isequal(l->cpuset, parent->cpuset)) {
@@ -111,11 +121,12 @@ output_topology (hwloc_topology_t topology, hwloc_obj_t l, hwloc_obj_t parent, F
     indent (output, 2*i);
     i++;
   }
-  output_console_obj(l, output, logical, verbose_mode);
+  output_console_obj(topology, l, output, logical, verbose_mode);
   if (l->arity || (!i && !l->arity))
     {
       for (x=0; x<l->arity; x++)
-	output_topology (topology, l->children[x], l, output, i, logical, verbose_mode);
+	if (l->children[x]->type != HWLOC_OBJ_PU || !lstopo_ignore_pus)
+	  output_topology (topology, l->children[x], l, output, i, logical, verbose_mode);
   }
 }
 
@@ -124,8 +135,8 @@ static void
 output_only (hwloc_topology_t topology, hwloc_obj_t l, FILE *output, int logical, int verbose_mode)
 {
   unsigned x;
-  if (show_only == l->type) {
-    output_console_obj (l, output, logical, verbose_mode);
+  if (lstopo_show_only == l->type) {
+    output_console_obj (topology, l, output, logical, verbose_mode);
     fprintf (output, "\n");
   }
   for (x=0; x<l->arity; x++)
@@ -155,39 +166,20 @@ void output_console(hwloc_topology_t topology, const char *filename, int logical
    * if verbose_mode > 1, print both.
    */
 
-  if (show_only != (hwloc_obj_type_t)-1) {
+  if (lstopo_show_only != (hwloc_obj_type_t)-1) {
     if (verbose_mode > 1)
-      fprintf(output, "Only showing %s objects\n", hwloc_obj_type_string(show_only));
+      fprintf(output, "Only showing %s objects\n", hwloc_obj_type_string(lstopo_show_only));
     output_only (topology, hwloc_get_root_obj(topology), output, logical, verbose_mode);
   } else if (verbose_mode >= 1) {
     output_topology (topology, hwloc_get_root_obj(topology), NULL, output, 0, logical, verbose_mode);
     fprintf(output, "\n");
   }
 
-  if (verbose_mode > 1 || !verbose_mode) {
-    unsigned depth, nbobjs;
-    for (depth = 0; depth < topodepth; depth++) {
-      hwloc_obj_type_t type = hwloc_get_depth_type (topology, depth);
-      nbobjs = hwloc_get_nbobjs_by_depth (topology, depth);
-      indent(output, depth);
-      fprintf (output, "depth %u:\t%u %s%s (type #%u)\n",
-	       depth, nbobjs, hwloc_obj_type_string (type), nbobjs>1?"s":"", type);
-    }
-    nbobjs = hwloc_get_nbobjs_by_depth (topology, HWLOC_TYPE_DEPTH_BRIDGE);
-    if (nbobjs)
-      fprintf (output, "Special depth %d:\t%u %s%s (type #%u)\n",
-	       HWLOC_TYPE_DEPTH_BRIDGE, nbobjs, "Bridge", nbobjs>1?"s":"", HWLOC_OBJ_BRIDGE);
-    nbobjs = hwloc_get_nbobjs_by_depth (topology, HWLOC_TYPE_DEPTH_PCI_DEVICE);
-    if (nbobjs)
-      fprintf (output, "Special depth %d:\t%u %s%s (type #%u)\n",
-	       HWLOC_TYPE_DEPTH_PCI_DEVICE, nbobjs, "PCI Device", nbobjs>1?"s":"", HWLOC_OBJ_PCI_DEVICE);
-    nbobjs = hwloc_get_nbobjs_by_depth (topology, HWLOC_TYPE_DEPTH_OS_DEVICE);
-    if (nbobjs)
-      fprintf (output, "Special depth %d:\t%u %s%s (type #%u)\n",
-	       HWLOC_TYPE_DEPTH_OS_DEVICE, nbobjs, "OS Device", nbobjs>1?"s":"", HWLOC_OBJ_OS_DEVICE);
-  }
+  if ((verbose_mode > 1 || !verbose_mode) && lstopo_show_only == (hwloc_obj_type_t)-1) {
+    hwloc_lstopo_show_summary(output, topology);
+ }
 
-  if (verbose_mode > 1) {
+  if (verbose_mode > 1 && lstopo_show_only == (hwloc_obj_type_t)-1) {
     const struct hwloc_distances_s * distances;
     unsigned depth;
 
@@ -203,7 +195,7 @@ void output_console(hwloc_topology_t topology, const char *filename, int logical
     }
   }
 
-  if (verbose_mode > 1) {
+  if (verbose_mode > 1 && lstopo_show_only == (hwloc_obj_type_t)-1) {
     hwloc_const_bitmap_t complete = hwloc_topology_get_complete_cpuset(topology);
     hwloc_const_bitmap_t topo = hwloc_topology_get_topology_cpuset(topology);
     hwloc_const_bitmap_t online = hwloc_topology_get_online_cpuset(topology);
@@ -265,6 +257,12 @@ void output_synthetic(hwloc_topology_t topology, const char *filename, int logic
   hwloc_obj_t obj = hwloc_get_root_obj(topology);
   int arity;
 
+  if (!obj->symmetric_subtree) {
+    fprintf(stderr, "Cannot output assymetric topology in synthetic format.\n");
+    fprintf(stderr, "Adding --no-io may help making the topology symmetric.\n");
+    return;
+  }
+
   if (!filename || !strcmp(filename, "-"))
     output = stdout;
   else {
@@ -275,11 +273,6 @@ void output_synthetic(hwloc_topology_t topology, const char *filename, int logic
     }
   }
 
-  if (!obj->symmetric_subtree) {
-    fprintf(stderr, "Cannot output assymetric topology in synthetic format\n");
-    return;
-  }
-
   arity = obj->arity;
   while (arity) {
     obj = obj->first_child;
@@ -287,6 +280,9 @@ void output_synthetic(hwloc_topology_t topology, const char *filename, int logic
     arity = obj->arity;
   }
   fprintf(output, "\n");
+
+  if (output != stdout)
+    fclose(output);
 }
 
 /*
