@@ -1,7 +1,7 @@
 /*
  * Copyright © 2009 CNRS
  * Copyright © 2009 inria.  All rights reserved.
- * Copyright © 2009-2010 Université Bordeaux 1
+ * Copyright © 2009-2010, 2012 Université Bordeaux 1
  * Copyright © 2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
  */
@@ -10,7 +10,9 @@
 
 #include <stdlib.h>
 #include <stdio.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <string.h>
 
 #include <windows.h>
@@ -195,15 +197,16 @@ WndProc(HWND hwnd, UINT message, WPARAM wparam, LPARAM lparam)
 static void *
 windows_start(void *output_ __hwloc_attribute_unused, int width, int height)
 {
-  WNDCLASS wndclass = {
-    .hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH),
-    .hCursor = LoadCursor(NULL, IDC_SIZEALL),
-    .hIcon = LoadIcon(NULL, IDI_APPLICATION),
-    .lpfnWndProc = WndProc,
-    .lpszClassName = "lstopo",
-  };
+  WNDCLASS wndclass;
   HWND toplevel;
 
+  memset(&wndclass, 0, sizeof(wndclass));
+  wndclass.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
+  wndclass.hCursor = LoadCursor(NULL, IDC_SIZEALL);
+  wndclass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
+  wndclass.lpfnWndProc = WndProc;
+  wndclass.lpszClassName = "lstopo";
+
   win_width = width + 2*GetSystemMetrics(SM_CXSIZEFRAME);
   win_height = height + 2*GetSystemMetrics(SM_CYSIZEFRAME) + GetSystemMetrics(SM_CYCAPTION);
 
@@ -291,12 +294,12 @@ void
 output_windows (hwloc_topology_t topology, const char *filename __hwloc_attribute_unused, int logical, int legend, int verbose_mode __hwloc_attribute_unused)
 {
   HWND toplevel;
+  MSG msg;
   the_topology = topology;
   the_logical = logical;
   the_legend = legend;
   toplevel = output_draw_start(&windows_draw_methods, logical, legend, topology, NULL);
   UpdateWindow(toplevel);
-  MSG msg;
   while (!finish && GetMessage(&msg, NULL, 0, 0)) {
     TranslateMessage(&msg);
     DispatchMessage(&msg);
