@@ -1,6 +1,6 @@
 /*
  * Copyright © 2009 CNRS
- * Copyright © 2009-2010 inria.  All rights reserved.
+ * Copyright © 2009-2013 Inria.  All rights reserved.
  * Copyright © 2009-2012 Université Bordeaux 1
  * Copyright © 2009-2011 Cisco Systems, Inc.  All rights reserved.
  * See COPYING in top-level directory.
@@ -12,7 +12,9 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <assert.h>
 
 extern void usage(const char *name, FILE *where);
@@ -142,20 +144,21 @@ hwloc_utils_enable_input_format(struct hwloc_topology *topology,
     int err;
     err = stat(input, &inputst);
     if (err < 0) {
-      if (verbose)
+      if (verbose > 0)
 	printf("assuming `%s' is a synthetic topology description\n", input);
       input_format = HWLOC_UTILS_INPUT_SYNTHETIC;
     } else if (S_ISDIR(inputst.st_mode)) {
-      if (verbose)
+      if (verbose > 0)
 	printf("assuming `%s' is a file-system root\n", input);
       input_format = HWLOC_UTILS_INPUT_FSROOT;
     } else if (S_ISREG(inputst.st_mode)) {
-      if (verbose)
+      if (verbose > 0)
 	printf("assuming `%s' is a XML file\n", input);
       input_format = HWLOC_UTILS_INPUT_XML;
     } else {
       fprintf (stderr, "Unrecognized input file: %s\n", input);
       usage (callname, stderr);
+      return EXIT_FAILURE;
     }
   }
 
@@ -227,3 +230,50 @@ hwloc_utils_print_distance_matrix(hwloc_topology_t topology, hwloc_obj_t root, u
   }
 }
 
+static __hwloc_inline hwloc_pid_t
+hwloc_pid_from_number(int pid_number, int set_info __hwloc_attribute_unused)
+{
+  hwloc_pid_t pid;
+#ifdef HWLOC_WIN_SYS
+  pid = OpenProcess(set_info ? PROCESS_SET_INFORMATION : PROCESS_QUERY_INFORMATION, FALSE, pid_number);
+  if (!pid) {
+    DWORD error = GetLastError();
+    char *message;
+    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
+                  NULL, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (char *)&message, 0, NULL);
+    fprintf(stderr, "OpenProcess %d failed %ld: %s\n", pid_number, error, message);
+    exit(EXIT_FAILURE);
+  }
+#else
+  pid = pid_number;
+#endif
+  return pid;
+}
+
+static __hwloc_inline void
+hwloc_lstopo_show_summary(FILE *output, hwloc_topology_t topology)
+{
+  unsigned topodepth = hwloc_topology_get_depth(topology);
+  unsigned depth, nbobjs;
+  for (depth = 0; depth < topodepth; depth++) {
+    hwloc_obj_t obj = hwloc_get_obj_by_depth(topology, depth, 0);
+    char type[64];
+    nbobjs = hwloc_get_nbobjs_by_depth (topology, depth);
+    fprintf(output, "%*s", (int) depth, "");
+    hwloc_obj_type_snprintf(type, sizeof(type), obj, 1);
+    fprintf (output,"depth %u:\t%u %s (type #%u)\n",
+	     depth, nbobjs, type, obj->type);
+  }
+  nbobjs = hwloc_get_nbobjs_by_depth (topology, HWLOC_TYPE_DEPTH_BRIDGE);
+  if (nbobjs)
+    fprintf (output, "Special depth %d:\t%u %s (type #%u)\n",
+	     HWLOC_TYPE_DEPTH_BRIDGE, nbobjs, "Bridge", HWLOC_OBJ_BRIDGE);
+  nbobjs = hwloc_get_nbobjs_by_depth (topology, HWLOC_TYPE_DEPTH_PCI_DEVICE);
+  if (nbobjs)
+    fprintf (output, "Special depth %d:\t%u %s (type #%u)\n",
+	     HWLOC_TYPE_DEPTH_PCI_DEVICE, nbobjs, "PCI Device", HWLOC_OBJ_PCI_DEVICE);
+  nbobjs = hwloc_get_nbobjs_by_depth (topology, HWLOC_TYPE_DEPTH_OS_DEVICE);
+  if (nbobjs)
+    fprintf (output, "Special depth %d:\t%u %s (type #%u)\n",
+	     HWLOC_TYPE_DEPTH_OS_DEVICE, nbobjs, "OS Device", HWLOC_OBJ_OS_DEVICE);
+}
